@@ -2,12 +2,14 @@ import os
 from copy import deepcopy
 
 from pyfileconf import Selector, PipelineManager
+from pyfileconf.batch import BatchUpdater
 from pyfileconf.sectionpath.sectionpath import SectionPath
 from pyfileconf.context import context
-from tests.input_files.amodule import SecondExampleClass
-from tests.input_files.mypackage.cmodule import ExampleClass
-from tests.test_pipeline_manager.base import PipelineManagerTestBase, CLASS_CONFIG_DICT_LIST, SAME_CLASS_CONFIG_DICT_LIST, \
-    DIFFERENT_CLASS_CONFIG_DICT_LIST
+from tests.input_files.amodule import SecondExampleClass, a_function
+from tests.input_files.mypackage.cmodule import ExampleClass, ExampleClassWithCustomUpdate
+from tests.test_pipeline_manager.base import PipelineManagerTestBase, CLASS_CONFIG_DICT_LIST, \
+    SAME_CLASS_CONFIG_DICT_LIST, \
+    DIFFERENT_CLASS_CONFIG_DICT_LIST, CUSTOM_UPDATE_CLASS_CONFIG_DICT_LIST
 
 
 class TestPipelineManagerConfig(PipelineManagerTestBase):
@@ -129,6 +131,33 @@ class TestPipelineManagerConfig(PipelineManagerTestBase):
         )
         assert context.config_dependencies == self.expect_pm_1_a_function_depends_on_pm_2_a_function
 
+    def test_config_update_batch_functions(self):
+        self.write_a_function_to_pipeline_dict_file()
+        pipeline_manager = self.create_pm()
+        pipeline_manager.load()
+        pipeline_manager.create('stuff2', a_function)
+        sel = Selector()
+        ivs = [
+            sel.test_pipeline_manager.stuff.a_function,
+            sel.test_pipeline_manager.stuff2.a_function,
+        ]
+        expected_b_result = ['a', 'b']
+        updates = []
+        for iv in ivs:
+            section_path = SectionPath.from_section_str_list(SectionPath(iv.section_path_str)[1:])
+            updates.append(dict(
+                b=expected_b_result,
+                section_path_str=section_path.path_str
+            ))
+        pipeline_manager.update_batch(updates)
+        for iv in ivs:
+            section_path = SectionPath.from_section_str_list(SectionPath(iv.section_path_str)[1:])
+            result = pipeline_manager.run(iv)
+            assert result == (None, expected_b_result)
+            pipeline_manager.refresh(section_path.path_str)
+            result = pipeline_manager.run(iv)
+            assert result == (None, expected_b_result)
+
     def test_config_update_class(self):
         self.write_example_class_to_pipeline_dict_file()
         pipeline_manager = self.create_pm()
@@ -152,6 +181,31 @@ class TestPipelineManagerConfig(PipelineManagerTestBase):
         assert ec == ExampleClass(expected_a_result)
         assert ec._f == expected_f_value
 
+    def test_config_update_class_with_custom_update(self):
+        self.write_example_class_with_custom_update_to_pipeline_dict_file()
+        pipeline_manager = self.create_pm()
+        pipeline_manager.load()
+        sel = Selector()
+        iv = sel.test_pipeline_manager.stuff.ExampleClassWithCustomUpdate
+        expected_a_result = (1, 2)
+        expected_f_value = 'woo'
+        assert iv._custom_update == 'a'
+        section_path = SectionPath.from_section_str_list(SectionPath(iv.section_path_str)[1:])
+        pipeline_manager.update(
+            a=expected_a_result,
+            section_path_str=section_path.path_str
+        )
+        ec = sel.test_pipeline_manager.stuff.ExampleClassWithCustomUpdate()
+        ec._f = expected_f_value
+        ec = sel.test_pipeline_manager.stuff.ExampleClassWithCustomUpdate()
+        assert ec == ExampleClassWithCustomUpdate(expected_a_result)
+        assert ec._f == expected_f_value
+        assert ec._custom_update == 'b'
+        pipeline_manager.refresh(section_path.path_str)
+        ec = sel.test_pipeline_manager.stuff.ExampleClassWithCustomUpdate()
+        assert ec == ExampleClassWithCustomUpdate(expected_a_result)
+        assert ec._f == expected_f_value
+
     def test_config_update_by_file_for_class(self):
         self.write_example_class_to_pipeline_dict_file()
         pipeline_manager = self.create_pm()
@@ -255,6 +309,38 @@ class TestPipelineManagerConfig(PipelineManagerTestBase):
         )
         assert context.config_dependencies == self.expect_pm_1_class_depends_on_pm_2_class
 
+    def test_config_update_batch_classes(self):
+        self.write_example_class_to_pipeline_dict_file()
+        pipeline_manager = self.create_pm()
+        pipeline_manager.load()
+        pipeline_manager.create('stuff2', ExampleClass)
+        sel = Selector()
+        ivs = [
+            sel.test_pipeline_manager.stuff.ExampleClass,
+            sel.test_pipeline_manager.stuff2.ExampleClass,
+        ]
+        expected_a_result = (1, 2)
+        expected_f_value = 'woo'
+        updates = []
+        for iv in ivs:
+            section_path = SectionPath.from_section_str_list(SectionPath(iv.section_path_str)[1:])
+            updates.append(dict(
+                a=expected_a_result,
+                section_path_str=section_path.path_str
+            ))
+        pipeline_manager.update_batch(updates)
+        for iv in ivs:
+            section_path = SectionPath.from_section_str_list(SectionPath(iv.section_path_str)[1:])
+            ec = sel.test_pipeline_manager.stuff.ExampleClass()
+            ec._f = expected_f_value
+            ec = sel.test_pipeline_manager.stuff.ExampleClass()
+            assert ec == ExampleClass(expected_a_result)
+            assert ec._f == expected_f_value
+            pipeline_manager.refresh(section_path.path_str)
+            ec = sel.test_pipeline_manager.stuff.ExampleClass()
+            assert ec == ExampleClass(expected_a_result)
+            assert ec._f == expected_f_value
+
     def test_create_update_from_specific_class_dict(self):
         self.write_example_class_dict_to_file()
         pipeline_manager = self.create_pm(
@@ -271,7 +357,7 @@ class TestPipelineManagerConfig(PipelineManagerTestBase):
             section_path_str=section_path.path_str
         )
         ec = sel.test_pipeline_manager.example_class.stuff.data
-        # TODO: setting attribute on selector should set attribute on underlying object
+        # TODO [#108]: setting attribute on selector should set attribute on underlying object
         #
         # Modify the next line in this test to `ec._f = expected_f_result` and
         # it will show the need for this behavior
@@ -288,6 +374,37 @@ class TestPipelineManagerConfig(PipelineManagerTestBase):
         assert ec.a == ec._a == expect_ec.a
         assert ec._f == expected_f_result
 
+    def test_create_update_from_specific_class_dict_with_custom_update(self):
+        self.write_example_class_dict_to_file(3)  # index 3 is with custom update
+        pipeline_manager = self.create_pm(
+            specific_class_config_dicts=CUSTOM_UPDATE_CLASS_CONFIG_DICT_LIST
+        )
+        pipeline_manager.load()
+        sel = Selector()
+        iv = sel.test_pipeline_manager.example_class_with_update.stuff.data
+        expected_a_result = (1, 2)
+        expected_f_result = 'woo'
+        section_path = SectionPath.from_section_str_list(SectionPath(iv.section_path_str)[1:])
+        pipeline_manager.update(
+            a=expected_a_result,
+            section_path_str=section_path.path_str
+        )
+        ec = sel.test_pipeline_manager.example_class_with_update.stuff.data
+        ec.item._f = expected_f_result
+        ec = sel.test_pipeline_manager.example_class_with_update.stuff.data
+        expect_ec = ExampleClass(name='data', a=expected_a_result)
+        assert ec.name == expect_ec.name
+        assert ec.a == expect_ec.a
+        assert ec._f == expected_f_result
+        assert ec._custom_update == 'b'
+        pipeline_manager.refresh(section_path.path_str)
+        ec = sel.test_pipeline_manager.example_class_with_update.stuff.data
+        expect_ec = ExampleClass(name='data', a=expected_a_result)
+        assert ec.name == expect_ec.name
+        assert ec.a == expect_ec.a
+        assert ec._f == expected_f_result
+        assert ec._custom_update == 'b'
+
     def test_config_update_by_file_for_specific_class_dict(self):
         self.write_example_class_dict_to_file()
         pipeline_manager = self.create_pm(
@@ -499,6 +616,44 @@ class TestPipelineManagerConfig(PipelineManagerTestBase):
         assert ec.name == sec.name == expect_ec.name
         assert ec.a == ec._a == sec.b == expect_ec.a == expect_sec.b
 
+    def test_create_update_batch_from_specific_class_dict(self):
+        self.write_example_class_dict_to_file()
+        pipeline_manager = self.create_pm(
+            specific_class_config_dicts=CLASS_CONFIG_DICT_LIST
+        )
+        pipeline_manager.load()
+        pipeline_manager.create('example_class.stuff.data2')
+        sel = Selector()
+        ivs = [
+            sel.test_pipeline_manager.example_class.stuff.data,
+            sel.test_pipeline_manager.example_class.stuff.data2,
+        ]
+        expected_a_result = (1, 2)
+        expected_f_result = 'woo'
+        updates = []
+        for iv in ivs:
+            section_path = SectionPath.from_section_str_list(SectionPath(iv.section_path_str)[1:])
+            updates.append(dict(
+                a=expected_a_result,
+                section_path_str=section_path.path_str
+            ))
+        pipeline_manager.update_batch(updates)
+        for ec in ivs:
+            section_path = SectionPath.from_section_str_list(SectionPath(ec.section_path_str)[1:])
+            ec = sel.test_pipeline_manager.example_class.stuff.data
+            ec.item._f = expected_f_result
+            ec = sel.test_pipeline_manager.example_class.stuff.data
+            expect_ec = ExampleClass(name='data', a=expected_a_result)
+            assert ec.name == expect_ec.name
+            assert ec.a == ec._a == expect_ec.a
+            assert ec._f == expected_f_result
+            pipeline_manager.refresh(section_path.path_str)
+            ec = sel.test_pipeline_manager.example_class.stuff.data
+            expect_ec = ExampleClass(name='data', a=expected_a_result)
+            assert ec.name == expect_ec.name
+            assert ec.a == ec._a == expect_ec.a
+            assert ec._f == expected_f_result
+
     def test_config_reload_function(self):
         self.write_a_function_to_pipeline_dict_file()
         pipeline_manager = self.create_pm()
@@ -515,6 +670,22 @@ class TestPipelineManagerConfig(PipelineManagerTestBase):
         result = pipeline_manager.run(iv)
         assert result == (None, None)
 
+    def test_config_reset_function(self):
+        self.write_a_function_to_pipeline_dict_file()
+        pipeline_manager = self.create_pm()
+        pipeline_manager.load()
+        sel = Selector()
+        iv = sel.test_pipeline_manager.stuff.a_function
+        expected_b_result = ['a', 'b']
+        section_path = SectionPath.from_section_str_list(SectionPath(iv.section_path_str)[1:])
+        pipeline_manager.update(
+            b=expected_b_result,
+            section_path_str=section_path.path_str
+        )
+        pipeline_manager.reset(section_path.path_str)
+        result = pipeline_manager.run(iv)
+        assert result == (None, None)
+
     def test_config_reload_function_multiple_pms(self):
         self.write_a_function_to_pipeline_dict_file()
         self.write_a_function_to_pipeline_dict_file(file_path=self.second_pipeline_dict_path)
@@ -567,6 +738,22 @@ class TestPipelineManagerConfig(PipelineManagerTestBase):
         ec = sel.test_pipeline_manager.stuff.ExampleClass()
         assert ec == ExampleClass(None)
 
+    def test_config_reset_class(self):
+        self.write_example_class_to_pipeline_dict_file()
+        pipeline_manager = self.create_pm()
+        pipeline_manager.load()
+        sel = Selector()
+        iv = sel.test_pipeline_manager.stuff.ExampleClass
+        expected_a_result = (1, 2)
+        section_path = SectionPath.from_section_str_list(SectionPath(iv.section_path_str)[1:])
+        pipeline_manager.update(
+            a=expected_a_result,
+            section_path_str=section_path.path_str
+        )
+        pipeline_manager.reset(section_path.path_str)
+        ec = sel.test_pipeline_manager.stuff.ExampleClass()
+        assert ec == ExampleClass(None)
+
     def test_config_reload_class_multiple_pms(self):
         self.write_example_class_to_pipeline_dict_file()
         self.write_example_class_to_pipeline_dict_file(file_path=self.second_pipeline_dict_path)
@@ -623,6 +810,26 @@ class TestPipelineManagerConfig(PipelineManagerTestBase):
         assert ec.name == expect_ec.name
         assert ec.a == ec._a == expect_ec.a
 
+    def test_config_reset_specific_class_dict(self):
+        self.write_example_class_dict_to_file()
+        pipeline_manager = self.create_pm(
+            specific_class_config_dicts=CLASS_CONFIG_DICT_LIST
+        )
+        pipeline_manager.load()
+        sel = Selector()
+        iv = sel.test_pipeline_manager.example_class.stuff.data
+        expected_a_result = (1, 2)
+        section_path = SectionPath.from_section_str_list(SectionPath(iv.section_path_str)[1:])
+        pipeline_manager.update(
+            a=expected_a_result,
+            section_path_str=section_path.path_str
+        )
+        pipeline_manager.reset(section_path.path_str)
+        ec = sel.test_pipeline_manager.example_class.stuff.data
+        expect_ec = ExampleClass(None, name='data')
+        assert ec.name == expect_ec.name
+        assert ec.a == ec._a == expect_ec.a
+
     def test_config_reload_specific_class_dict_multiple_pms(self):
         self.write_example_class_dict_to_file()
         self.write_example_class_dict_to_file(pm_index=1)
@@ -804,4 +1011,138 @@ class TestPipelineManagerConfig(PipelineManagerTestBase):
 
         # Config is only loaded once attribute of item view is accessed
         with self.assertRaises(ValueError):
-            iv.item
\ No newline at end of file
+            iv.item
+
+
+class TestBatchUpdater(PipelineManagerTestBase):
+
+    def test_config_batch_updater_function_multiple_pms(self):
+        self.write_a_function_to_pipeline_dict_file()
+        self.write_a_function_to_pipeline_dict_file(file_path=self.second_pipeline_dict_path)
+        pipeline_manager = self.create_pm()
+        pipeline_manager.load()
+        pipeline_manager2 = self.create_pm(
+            folder=self.second_pm_folder,
+            name=self.second_test_name,
+        )
+        pipeline_manager2.load()
+        sel = Selector()
+        expected_b_result = ['a', 'b']
+        pms = [
+            pipeline_manager,
+            pipeline_manager2,
+        ]
+        ivs = [
+            sel.test_pipeline_manager.stuff.a_function,
+            sel.test_pipeline_manager2.stuff.a_function
+        ]
+
+        updates = []
+        for iv in ivs:
+            updates.append(dict(
+                b=expected_b_result,
+                section_path_str=iv.section_path_str
+            ))
+
+        bu = BatchUpdater()
+        bu.update(updates)
+
+        for pm, iv in zip(pms, ivs):
+            result = pm.run(iv)
+            assert result == (None, expected_b_result)
+
+        sp_strs = [conf['section_path_str'] for conf in updates]
+        bu.reset(sp_strs)
+
+        for pm, iv in zip(pms, ivs):
+            result = pm.run(iv)
+            assert result == (None, None)
+
+    def test_batch_updater_class_multiple_pms(self):
+        self.write_example_class_to_pipeline_dict_file()
+        self.write_example_class_to_pipeline_dict_file(file_path=self.second_pipeline_dict_path)
+        pipeline_manager = self.create_pm()
+        pipeline_manager.load()
+        pipeline_manager2 = self.create_pm(
+            folder=self.second_pm_folder,
+            name=self.second_test_name,
+        )
+        pipeline_manager2.load()
+        pms = [
+            pipeline_manager,
+            pipeline_manager2,
+        ]
+        sel = Selector()
+        expected_a_result = (1, 2)
+        ivs = [
+            sel.test_pipeline_manager.stuff.ExampleClass,
+            sel.test_pipeline_manager2.stuff.ExampleClass,
+        ]
+
+        updates = []
+        for iv in ivs:
+            updates.append(dict(
+                a=expected_a_result,
+                section_path_str=iv.section_path_str
+            ))
+
+        bu = BatchUpdater()
+        bu.update(updates)
+
+        for iv in ivs:
+            ec = iv()
+            assert ec == ExampleClass(expected_a_result)
+
+        sp_strs = [conf['section_path_str'] for conf in updates]
+        bu.reset(sp_strs)
+
+        for iv in ivs:
+            ec = iv()
+            assert ec == ExampleClass(None)
+
+    def test_batch_updater_specific_class_dict_multiple_pms(self):
+        self.write_example_class_dict_to_file()
+        self.write_example_class_dict_to_file(pm_index=1)
+        pipeline_manager = self.create_pm(
+            specific_class_config_dicts=CLASS_CONFIG_DICT_LIST
+        )
+        pipeline_manager.load()
+        pipeline_manager2 = self.create_pm(
+            folder=self.second_pm_folder,
+            name=self.second_test_name,
+            specific_class_config_dicts=CLASS_CONFIG_DICT_LIST,
+        )
+        pipeline_manager2.load()
+        pms = [
+            pipeline_manager,
+            pipeline_manager2,
+        ]
+        sel = Selector()
+        expected_a_result = (1, 2)
+        ivs = [
+            sel.test_pipeline_manager.example_class.stuff.data,
+            sel.test_pipeline_manager2.example_class.stuff.data,
+        ]
+
+        updates = []
+        for iv in ivs:
+            updates.append(dict(
+                a=expected_a_result,
+                section_path_str=iv.section_path_str
+            ))
+
+        bu = BatchUpdater()
+        bu.update(updates)
+
+        for ec in ivs:
+            expect_ec = ExampleClass(name='data', a=expected_a_result)
+            assert ec.name == expect_ec.name
+            assert ec.a == ec._a == expect_ec.a
+
+        sp_strs = [conf['section_path_str'] for conf in updates]
+        bu.reset(sp_strs)
+
+        for ec in ivs:
+            expect_ec = ExampleClass(name='data', a=None)
+            assert ec.name == expect_ec.name
+            assert ec.a == ec._a == expect_ec.a
\ No newline at end of file
